/*****************************************************************************
 *                                                                           *
 *     Header for the serial I/O class                                       *
 *     Version 1.1                        Author      :    Henry Koplien     *
 *                                        Create date :    19.11.98          *
 *                                        Last edit   :    29.11.98          *
 *****************************************************************************/

#define RX_CRC_INIT     0xffff
#define TX_CRC_INIT     0xffff
#define SERIAL_DEVICE   "cuc"
#define LOCK_PREFIX     "/usr/spool/uucp/LCK/LCK.."

/**
  *  Used to present a unique serial interface to the world.
  *  It is an object taylor made for BSD on NeXT. If it is necessary to port
  *  it elsewhere feel free to do so... NeXT handles some things different
  *  as the LOCK file and some other things. The only things handled are
  *  sending and receiving bytes through the serial I/O. What is an addition
  *  is the crc checksum on the fly. This could take place in the protocol
  *  but was left for the basics to ensure a reliable transmission.
  *  Something that is still missing in version 1.0 is a good treatment
  *  of a timeout and a reconnection of the line...
  *  What is also in this package is the signal handler serving a lot
  *  of different signals. What was added in version 1.1 is the method
  *  {\em Activities}, to remove all the nasty syslog stuff out of the signal
  *  handler. Nevertheless is this the right place for a future extension.
  *
  *  @author  Henry Koplien
  *  @version 1.1 (first release 19. November 1998, last edit 29. November 1998)
  */
class rs232: public chain {

private:

  /// Actual crc value for transmit
  unsigned short int    tx_crc;
  /// Actual crc value for receive
  unsigned short int    rx_crc;
  /// File descriptor used as a handle
  int                   fd;
  /// Flag for signaling a lock file
  bool                  locked;
 
  /**
    *  CRC function.
    *  This method is a functional copy of the CRC checksum algorithm from the
    *  TI-99/4A of the good old days of Texas Instruments.
    *
    *  @param  crc_value The crc value before taking {\bf data} into account
    *  @param  data      The data to take into account
    *  @return           The calculated crc checksum
    */
  unsigned short int    crc                   (unsigned short int crc_value,
                                               unsigned char data);
  /**
    *  Remove the lock file.
    *  This method is an internal one and must be called before an error
    *  message.
    */
  void                  remove_lock           (void);

public:

  /**
    *  Toggle the RTS line.
    *  If the RTS line is low, the line is risen high for 5 seconds,
    *  signaling the UPS to shutdown in the near future, if the UPS is
    *  on battery. Otherwise the UPS ignores the line toggle.
    */
  void                  RTS_toggle            (void);
  /**
    *  Look for a ring alarm.
    *  If the RNG line is set for an amount of time the method returns
    *  {\bf TRUE}. If the battery is low, the alarm is set also.
    *
    *  @return      TRUE or FALSE, depends on the RNG time and the battery
    *               status
    */
  bool                  Ring_Alarm            (void);
  /**
    *  Returns the status if there was a special signal received.
    *  The signals to look for are only various stop signals, which will
    *  normally terminate a program.
    *
    *  @return      TRUE or FALSE, depends on status of the received
    *               signals
    */
  bool                  Signal_Escape         (void);
  /**
    *  Looks for activities since the last call.
    *  This method is used to show the activities without using things
    *  like syslog in the signal handler. It is therefore possible to
    *  have all the nasty syslog stuff in the {\bf sys\_message} object.
    */
  void                  Activities            (void);
  /**
    *  Cycle redundancy check prepare for transmit.
    *  This is the right place to feed the CRC with a special value for 
    *  sending.
    */
  void                  tx_crc_init           (void);
  /**
    *  Cycle redundancy check prepare for receive.
    *  This is the right place to feed the CRC with a special value for
    *  receiving.
    */
  void                  rx_crc_init           (void);
  /**
    *  Verify the received data.
    *  Simple check of the crc data to zero.
    *
    *  @return      TRUE or FALSE, depends on status of received data
    */
  bool                  rx_ok                 (void);
  /**
    *  Sends a byte through the interface.
    *  Very evident function sending one single byte. Makes a CRC on the fly.
    *  This function is used to transmit data's. 
    *
    *  @param  data The data to take into account
    *  @return      TRUE or FALSE, depends on the transmit status
    */
  bool                  send                  (const unsigned char data);
  /**
    *  Sends the crc data of the transmitted bytes.
    *  This function is used to transmit the crc data's since last
    *  initialization. 
    *
    *  @return      TRUE or FALSE, depends on the transmit status of the crc
    */
  bool                  send_crc              (void);
  /**
    *  Receives a byte from the serial interface.
    *  Very evident function receiving one single byte. Makes a CRC on the fly.
    *  This function is used to receive data's. Optional parameter {\em n} is
    *  used as a minimum of requested bytes. 
    *
    *  @param  cptr The pointer where to write the received data
    *  @param  n    Number of desired bytes, default is one
    *  @return      TRUE on success, otherwise FALSE
    */
  bool                  receive               (unsigned char *cptr, int n=1);
  /**
    *  Sniffing the interface.
    *  Enables the watch dog timer and have an eye on line breaks etc.
    */
  void                  enable_watchdog       (void);
  /**
    *  Sniffing the interface is switched off.
    *  Disables the watch dog timer and let things keep going.
    */
  void                  disable_watchdog      (void);

public:

  /**
    *  The constructor.
    *  Initialize the own data and look for the LOCK file. Tries to open
    *  the fixed device and disappear if no device is available, makes
    *  a lock file, initialize the signal hander and does other neat stuff.
    */
                        rs232                 (void);
  /**
    *  The destructor.
    *  Release the device and and remove the LOCK file, switch off the
    *  signal handler.
    */
                        ~rs232                (void);
};
